import 'package:example/main.dart';
import 'package:example/widgets.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:routemaster/routemaster.dart';

class RouteMasterApp extends StatefulWidget {
  RouteMasterApp({Key? key}) : super(key: key);

  @override
  State<RouteMasterApp> createState() => _RouteMasterAppState();
}

class _RouteMasterAppState extends State<RouteMasterApp> {
  late final delegate = RoutemasterDelegate(routesBuilder: (_) {
    final paths = keyboardMappings.values.toList()..remove('/');
    Map<String, PageBuilder> routes = {};
    // Inject a bunch of routes, based on paths defined in keyboardMappings
    for (var p in paths) {
      routes[p] = (_) => MaterialPage(key: ValueKey(p), child: InstructionsScaffold(p));
    }
    // Use a _AppScaffold widget to wrap all the child routes in a shared scaffold
    routes['/'] = (_) => TabPage(child: _AppScaffold(), paths: paths);
    return RouteMap(routes: routes);
  });

  @override
  Widget build(BuildContext context) => KeyboardTestHarness(
        setLocation: (location) => delegate.push(location),
        child: MaterialApp.router(
          routeInformationParser: const RoutemasterParser(),
          routerDelegate: delegate,
        ),
      );
}

class _AppScaffold extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    final tabPage = TabPage.of(context);
    return Row(
      children: [
        const SideBar(),
        Expanded(
          child: TabBarView(
            controller: tabPage.controller,
            children: [
              for (final stack in tabPage.stacks) PageStackNavigator(stack: stack),
            ],
          ),
        ),
      ],
    );
  }
}
